// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.name.operation.SwapSynonymAndAcceptedOperation;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;

/**
 * <p>SwapSynonymAndAcceptedHandler class.</p>
 *
 * @author n.hoffmann
 * @created 21.04.2009
 * @version 1.0
 */
public class SwapSynonymAndAcceptedHandler extends AbstractHandler implements
		IHandler, IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(SwapSynonymAndAcceptedHandler.class);
	private MultiPageTaxonEditor editor;
	private Taxon taxon;

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	@Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
		editor = EditorUtil.getActiveMultiPageTaxonEditor();
		Shell shell = HandlerUtil.getActiveShell(event);
		Synonym synonym = (Synonym) EditorUtil.getSelection(event).getFirstElement();


		// Force user to save taxon - not really necessary though, is it?
		if (!EditorUtil.forceUserSave(editor, shell)) {
			return null;
		}

		try {
			SwapSynonymAndAcceptedOperation operation = new SwapSynonymAndAcceptedOperation(event.getCommand().getName(),
			        editor.getUndoContext(),
			        editor.getTaxon(),
			        synonym,
			        this,
			        (ICdmEntitySessionEnabled)editor.getEditorInput());

			EditorUtil.executeOperation(operation);

		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}

		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase, boolean)
	 */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		// Redraw existing editor
		//((IPostOperationEnabled) editor).postOperation(null);

		editor.doSave(EditorUtil.getMonitor());
		editor.close(true);

		if (objectAffectedByOperation instanceof Taxon) {

			taxon = (Taxon) objectAffectedByOperation;


		}
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#onComplete()
	 */
	@Override
	public boolean onComplete() {
		Display display = Display.getCurrent();
		display.asyncExec(new Runnable() {
			@Override
            public void run() {
				try {
					EditorUtil.openTaxonBase(taxon.getUuid());

				} catch (Exception e) {
					MessagingUtils.warningDialog("Could not open editor for taxon", this, e.getMessage());
				}

			}
		});
		return true;
	}
}
