/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <ol>
 * 	<li>Create synonym name using newSynonymText</li>
 * 	<li>Add name to new homotypic group</li>
 * 	<li>Create a new synonym for the taxon</li>
 * </ol>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 16.01.2009
 * @version 1.0
 */
public class CreateSynonymInNewGroupOperation extends AbstractPostTaxonOperation {

	// TODO replace this with TaxonNameBase
	private final TaxonNameBase newSynonymName;
	private SynonymRelationship newSynonymRelationship;

	/**
	 * <p>Constructor for CreateSynonymInNewGroupOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param newSynonymName a {@link eu.etaxonomy.cdm.model.name.TaxonNameBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateSynonymInNewGroupOperation(String label,
			IUndoContext undoContext, Taxon taxon, TaxonNameBase newSynonymName, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);

		this.newSynonymName = newSynonymName;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Add name to new homotypic group
		HomotypicalGroup.NewInstance().addTypifiedName(newSynonymName);
		monitor.worked(20);
		HibernateProxyHelper.deproxy(element);
		// Create a new synonym for the taxon
		newSynonymRelationship = element.addHeterotypicSynonymName(newSynonymName);
		monitor.worked(40);
		
		return postExecute(newSynonymRelationship.getSynonym());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Remove the synonym
		element.removeSynonymRelation(newSynonymRelationship);
		newSynonymRelationship = null;

		return postExecute(null);
	}
}
