// $Id$
/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Collection;

import org.apache.http.client.ClientProtocolException;

import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifQueryServiceWrapper;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifResponse;

/**
 * @author pplitzner
 * @date 27.05.2014
 *
 */
public class GbifImportEditorInput extends DataImportEditorInput<GbifResponse> {

    public GbifImportEditorInput(OccurenceQuery query) {
        super();
        this.query = query;
    }

    @Override
    public void query(){
        String errorMessage = "Could not execute query " + query;
        try{
            Collection<GbifResponse> results = new GbifQueryServiceWrapper().query(query);
            setResults(results);
        } catch (ClientProtocolException e) {
            logger.error(errorMessage, e);
        } catch (IOException e) {
            logger.error(errorMessage, e);
        } catch (URISyntaxException e) {
            logger.error(errorMessage, e);
        }
    }


    @Override
    public String getName() {
        return "[GBIF] " + query.toString();
    }

    @Override
    public String getToolTipText() {
        return "[GBIF] " + query.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((query == null) ? 0 : query.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        DataImportEditorInput<GbifResponse> other = (DataImportEditorInput<GbifResponse>) obj;
        if (query == null) {
            if (other.query != null) {
                return false;
            }
        } else if (!query.equals(other.query)) {
            return false;
        }
        return true;
    }

}
