// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport.transientServices;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.hibernate.LockOptions;
import org.hibernate.Session;
import org.hibernate.criterion.Criterion;
import org.hibernate.envers.query.criteria.AuditCriterion;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.config.CreateHierarchyForClassificationConfigurator;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.dto.FindByIdentifierDTO;
import eu.etaxonomy.cdm.api.service.pager.Pager;
import eu.etaxonomy.cdm.common.monitor.IProgressMonitor;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.ISourceable;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.LSID;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.view.AuditEvent;
import eu.etaxonomy.cdm.model.view.AuditEventRecord;
import eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort;
import eu.etaxonomy.cdm.persistence.dto.MergeResult;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.cdm.persistence.query.Grouping;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.cdm.persistence.query.OrderHint;
import eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.merge.IMergeStrategy;

/**
 * @author pplitzner
 * @date 27.09.2013
 *
 */
public class TransientClassificationService implements IClassificationService {

    private final IClassificationService defaultService;

    /**
     * @param defaultClassificationService
     */
    public TransientClassificationService(IClassificationService defaultClassificationService) {
        this.defaultService = defaultClassificationService;
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(eu.etaxonomy.cdm.model.common.VersionableEntity, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<Classification>> pageAuditEvents(Classification t, Integer pageSize, Integer pageNumber, AuditEventSort sort, List<String> propertyPaths) {
        return defaultService.pageAuditEvents(t, pageSize, pageNumber, sort, propertyPaths);
    }

    /**
     * @param annotatedObj
     * @param status
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getAnnotations(eu.etaxonomy.cdm.model.common.AnnotatableEntity, eu.etaxonomy.cdm.model.common.MarkerType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Annotation> getAnnotations(Classification annotatedObj, MarkerType status, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.getAnnotations(annotatedObj, status, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#getTaxonNodeByUuid(java.util.UUID)
     */
    @Override
    public TaxonNode getTaxonNodeByUuid(UUID uuid) {
        return defaultService.getTaxonNodeByUuid(uuid);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#getTreeNodeByUuid(java.util.UUID)
     */
    @Override
    public ITaxonTreeNode getTreeNodeByUuid(UUID uuid) {
        return defaultService.getTreeNodeByUuid(uuid);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IService#clear()
     */
    @Override
    public void clear() {
        defaultService.clear();
    }

    /**
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#listClassifications(java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Classification> listClassifications(Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.listClassifications(limit, start, orderHints, propertyPaths);
    }

    /**
     * @param t
     * @param lockMode
     * @see eu.etaxonomy.cdm.api.service.IService#lock(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode)
     */
    @Override
    public void lock(Classification t, LockOptions lockOptions) {
        defaultService.lock(t, lockOptions);
    }

    /**
     * @param t
     * @param lockMode
     * @param propertyPaths
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode, java.util.List)
     */
    @Override
    public void refresh(Classification t, LockOptions lockOptions, List<String> propertyPaths) {
        defaultService.refresh(t, lockOptions, propertyPaths);
    }

    /**
     * @param taxon
     * @param classificationUuid
     * @param propertyPaths
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#loadTaxonNodeByTaxon(eu.etaxonomy.cdm.model.taxon.Taxon, java.util.UUID, java.util.List)
     */
    @Deprecated
    @Override
    public TaxonNode loadTaxonNodeByTaxon(Taxon taxon, UUID classificationUuid, List<String> propertyPaths) {
        return defaultService.loadTaxonNodeByTaxon(taxon, classificationUuid, propertyPaths);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache()
     */
    @Override
    public void updateTitleCache() {
        defaultService.updateTitleCache();
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getNextAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<Classification> getNextAuditEvent(Classification t) {
        return defaultService.getNextAuditEvent(t);
    }

    /**
     * @param annotatableEntity
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getMarkers(eu.etaxonomy.cdm.model.common.AnnotatableEntity, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Marker> getMarkers(Classification annotatableEntity, Boolean technical, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.getMarkers(annotatableEntity, technical, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param stepSize
     * @param cacheStrategy
     * @param monitor
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache(java.lang.Class, java.lang.Integer, eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy, eu.etaxonomy.cdm.common.monitor.IProgressMonitor)
     */
    @Override
    public void updateTitleCache(Class<? extends Classification> clazz, Integer stepSize, IIdentifiableEntityCacheStrategy<Classification> cacheStrategy, IProgressMonitor monitor) {
        defaultService.updateTitleCache(clazz, stepSize, cacheStrategy, monitor);
    }

    /**
     * @param taxonNode
     * @param propertyPaths
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#loadTaxonNode(eu.etaxonomy.cdm.model.taxon.TaxonNode, java.util.List)
     */
    @Deprecated
    @Override
    public TaxonNode loadTaxonNode(TaxonNode taxonNode, List<String> propertyPaths) {
        return defaultService.loadTaxonNode(taxonNode, propertyPaths);
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getPreviousAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<Classification> getPreviousAuditEvent(Classification t) {
        return defaultService.getPreviousAuditEvent(t);
    }

    /**
     * @param clazz
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#count(java.lang.Class)
     */
    @Override
    public int count(Class<? extends Classification> clazz) {
        return defaultService.count(clazz);
    }

    /**
     * @param clazz
     * @param from
     * @param to
     * @param criteria
     * @param pageSize
     * @param pageValue
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(java.lang.Class, eu.etaxonomy.cdm.model.view.AuditEvent, eu.etaxonomy.cdm.model.view.AuditEvent, java.util.List, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<Classification>> pageAuditEvents(Class<? extends Classification> clazz, AuditEvent from, AuditEvent to, List<AuditCriterion> criteria, Integer pageSize, Integer pageValue, AuditEventSort sort, List<String> propertyPaths) {
        return defaultService.pageAuditEvents(clazz, from, to, criteria, pageSize, pageValue, sort, propertyPaths);
    }

    /**
     * @param lsid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#find(eu.etaxonomy.cdm.model.common.LSID)
     */
    @Override
    public Classification find(LSID lsid) {
        return defaultService.find(lsid);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#delete(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public DeleteResult delete(Classification persistentObject) {
        return defaultService.delete(persistentObject);
    }

    /**
     * @param clazz
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#groupMarkers(java.lang.Class, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public List<Object[]> groupMarkers(Class<? extends Classification> clazz, Boolean technical, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.groupMarkers(clazz, technical, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#exists(java.util.UUID)
     */
    @Override
    public boolean exists(UUID uuid) {
        return defaultService.exists(uuid);
    }

    /**
     * @param x
     * @param y
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#replace(eu.etaxonomy.cdm.model.common.IdentifiableEntity, eu.etaxonomy.cdm.model.common.IdentifiableEntity)
     */
    @Override
    public Classification replace(Classification x, Classification y) {
        return defaultService.replace(x, y);
    }

    /**
     * @param uuidSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.Set)
     */
    @Override
    public List<Classification> find(Set<UUID> uuidSet) {
        return defaultService.find(uuidSet);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSources(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<IdentifiableSource> getSources(Classification t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.getSources(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param classification
     * @param rank
     * @param pageSize
     * @param pageIndex
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#listRankSpecificRootNodes(eu.etaxonomy.cdm.model.taxon.Classification, eu.etaxonomy.cdm.model.name.Rank, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public List<TaxonNode> listRankSpecificRootNodes(Classification classification, Rank rank, Integer pageSize, Integer pageIndex, List<String> propertyPaths) {
        return defaultService.listRankSpecificRootNodes(classification, rank, pageSize, pageIndex, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.UUID)
     */
    @Override
    public Classification find(UUID uuid) {
        return defaultService.find(uuid);
    }

    /**
     * @param id
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(int)
     */
    @Override
    public Classification find(int id) {
        return defaultService.find(id);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getRights(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<Rights> getRights(Classification t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.getRights(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param clazz
     * @param technical
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#countMarkers(java.lang.Class, java.lang.Boolean)
     */
    @Override
    public int countMarkers(Class<? extends Classification> clazz, Boolean technical) {
        return defaultService.countMarkers(clazz, technical);
    }

    /**
     * @param idSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#findById(java.util.Set)
     */
    @Override
    public List<Classification> findById(Set<Integer> idSet) {
        return defaultService.findById(idSet);
    }

    /**
     * @param clazz
     * @param idInSource
     * @param idNamespace
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSourcedObjectByIdInSource(java.lang.Class, java.lang.String, java.lang.String)
     */
    @Override
    public ISourceable getSourcedObjectByIdInSource(Class clazz, String idInSource, String idNamespace) {
        return defaultService.getSourcedObjectByIdInSource(clazz, idInSource, idNamespace);
    }

    /**
     * @param classification
     * @param rank
     * @param pageSize
     * @param pageIndex
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#pageRankSpecificRootNodes(eu.etaxonomy.cdm.model.taxon.Classification, eu.etaxonomy.cdm.model.name.Rank, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<TaxonNode> pageRankSpecificRootNodes(Classification classification, Rank rank, Integer pageSize, Integer pageIndex, List<String> propertyPaths) {
        return defaultService.pageRankSpecificRootNodes(classification, rank, pageSize, pageIndex, propertyPaths);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#getSession()
     */
    @Override
    public Session getSession() {
        return defaultService.getSession();
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getUuidAndTitleCache()
     */
    @Override
    public List<UuidAndTitleCache<Classification>> getUuidAndTitleCache() {
        return defaultService.getUuidAndTitleCache();
    }

    /**
     * @param clazz
     * @param limit
     * @param start
     * @param groups
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#group(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Object[]> group(Class<? extends Classification> clazz, Integer limit, Integer start, List<Grouping> groups, List<String> propertyPaths) {
        return defaultService.group(clazz, limit, start, groups, propertyPaths);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Classification> findByTitle(Class<? extends Classification> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.findByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param taxonNode
     * @param baseRank
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#loadTreeBranch(eu.etaxonomy.cdm.model.taxon.TaxonNode, eu.etaxonomy.cdm.model.name.Rank, java.util.List)
     */
    @Override
    public List<TaxonNode> loadTreeBranch(TaxonNode taxonNode, Rank baseRank, List<String> propertyPaths) {
        return defaultService.loadTreeBranch(taxonNode, baseRank, propertyPaths);
    }

    /**
     * @param type
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <S extends Classification> List<S> list(Class<S> type, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.list(type, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param taxon
     * @param classification
     * @param baseRank
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#loadTreeBranchToTaxon(eu.etaxonomy.cdm.model.taxon.Taxon, eu.etaxonomy.cdm.model.taxon.Classification, eu.etaxonomy.cdm.model.name.Rank, java.util.List)
     */
    @Override
    public List<TaxonNode> loadTreeBranchToTaxon(Taxon taxon, Classification classification, Rank baseRank, List<String> propertyPaths) {
        return defaultService.loadTreeBranchToTaxon(taxon, classification, baseRank, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Pager<Classification> findByTitle(IIdentifiableEntityServiceConfigurator<Classification> configurator) {
        return defaultService.findByTitle(configurator);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List)
     */
    @Override
    public Integer countByTitle(Class<? extends Classification> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria) {
        return defaultService.countByTitle(clazz, queryString, matchmode, criteria);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID)
     */
    @Override
    public Classification load(UUID uuid) {
        return defaultService.load(uuid);
    }

    /**
     * @param uuid
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID, java.util.List)
     */
    @Override
    public Classification load(UUID uuid, List<String> propertyPaths) {
        return defaultService.load(uuid, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Integer countByTitle(IIdentifiableEntityServiceConfigurator<Classification> configurator) {
        return defaultService.countByTitle(configurator);
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#merge(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public Classification merge(Classification transientObject) {
        return defaultService.merge(transientObject);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Classification> listByTitle(Class<? extends Classification> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.listByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#page(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <S extends Classification> Pager<S> page(Class<S> type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.page(type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param taxonNode
     * @param propertyPaths
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#loadChildNodesOfTaxonNode(eu.etaxonomy.cdm.model.taxon.TaxonNode, java.util.List)
     */
    @Deprecated
    @Override
    public List<TaxonNode> loadChildNodesOfTaxonNode(TaxonNode taxonNode, List<String> propertyPaths) {
        return defaultService.loadChildNodesOfTaxonNode(taxonNode, propertyPaths);
    }

    /**
     * @param classification
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#getTaxonNodeUuidAndTitleCacheOfAcceptedTaxaByClassification(eu.etaxonomy.cdm.model.taxon.Classification)
     */
    @Override
    public List<UuidAndTitleCache<TaxonNode>> getTaxonNodeUuidAndTitleCacheOfAcceptedTaxaByClassification(Classification classification, List<UUID> excludeTaxa) {
        return defaultService.getTaxonNodeUuidAndTitleCacheOfAcceptedTaxaByClassification(classification, excludeTaxa);
    }

    /**
     * @param taxon
     * @param taxTree
     * @param propertyPaths
     * @param size
     * @param height
     * @param widthOrDuration
     * @param mimeTypes
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#getAllMediaForChildNodes(eu.etaxonomy.cdm.model.taxon.Taxon, eu.etaxonomy.cdm.model.taxon.Classification, java.util.List, int, int, int, java.lang.String[])
     */
    @Deprecated
    @Override
    public Map<UUID, List<MediaRepresentation>> getAllMediaForChildNodes(Taxon taxon, Classification taxTree, List<String> propertyPaths, int size, int height, int widthOrDuration, String[] mimeTypes) {
        return defaultService.getAllMediaForChildNodes(taxon, taxTree, propertyPaths, size, height, widthOrDuration, mimeTypes);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID refresh(Classification persistentObject) {
        return defaultService.refresh(persistentObject);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByReferenceTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Classification> listByReferenceTitle(Class<? extends Classification> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.listByReferenceTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param taxonNode
     * @param propertyPaths
     * @param size
     * @param height
     * @param widthOrDuration
     * @param mimeTypes
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#getAllMediaForChildNodes(eu.etaxonomy.cdm.model.taxon.TaxonNode, java.util.List, int, int, int, java.lang.String[])
     */
    @Override
    public Map<UUID, List<MediaRepresentation>> getAllMediaForChildNodes(TaxonNode taxonNode, List<String> propertyPaths, int size, int height, int widthOrDuration, String[] mimeTypes) {
        return defaultService.getAllMediaForChildNodes(taxonNode, propertyPaths, size, height, widthOrDuration, mimeTypes);
    }

    /**
     * @param tableName
     * @param limit
     * @param start
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#rows(java.lang.String, int, int)
     */
    @Override
    public List<Classification> rows(String tableName, int limit, int start) {
        return defaultService.rows(tableName, limit, start);
    }

    /**
     * @param newInstances
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(java.util.Collection)
     */
    @Override
    public Map<UUID, Classification> save(Collection<Classification> newInstances) {
        return null;
    }

    /**
     * @param taxonNode
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#removeTaxonNode(eu.etaxonomy.cdm.model.taxon.TaxonNode)
     */
    @Deprecated
    @Override
    public UUID removeTaxonNode(TaxonNode taxonNode) {
        return defaultService.removeTaxonNode(taxonNode);
    }

    /**
     * @param taxonNode
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#saveTaxonNode(eu.etaxonomy.cdm.model.taxon.TaxonNode)
     */
    @Deprecated
    @Override
    public UUID saveTaxonNode(TaxonNode taxonNode) {
        return null;
    }

    /**
     * @param newInstance
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public Classification save(Classification newInstance) {
        return null;
    }

    /**
     * @param taxonNodeCollection
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#saveTaxonNodeAll(java.util.Collection)
     */
    @Deprecated
    @Override
    public Map<UUID, TaxonNode> saveTaxonNodeAll(Collection<TaxonNode> taxonNodeCollection) {
        return null;
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID saveOrUpdate(Classification transientObject) {
        return null;
    }

    /**
     * @param treeNode
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#removeTreeNode(eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode)
     */
    @Override
    public UUID removeTreeNode(ITaxonTreeNode treeNode) {
        return defaultService.removeTreeNode(treeNode);
    }

    /**
     * @param clazz
     * @param queryString
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#search(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Classification> search(Class<? extends Classification> clazz, String queryString, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.search(clazz, queryString, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param treeNode
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#saveTreeNode(eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode)
     */
    @Override
    public UUID saveTreeNode(ITaxonTreeNode treeNode) {
        return null;
    }

    /**
     * @param transientObjects
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(java.util.Collection)
     */
    @Override
    public Map<UUID, Classification> saveOrUpdate(Collection<Classification> transientObjects) {
        return null;
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#getAllNodes()
     */
    @Override
    public List<TaxonNode> getAllNodes() {
        return defaultService.getAllNodes();
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#update(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID update(Classification transientObject) {
        return defaultService.update(transientObject);
    }

    /**
     * @param example
     * @param includeProperties
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(eu.etaxonomy.cdm.model.common.ICdmBase, java.util.Set, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Classification> list(Classification example, Set<String> includeProperties, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.list(example, includeProperties, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param matchStrategy
     * @param mergeStrategy
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#deduplicate(java.lang.Class, eu.etaxonomy.cdm.strategy.match.IMatchStrategy, eu.etaxonomy.cdm.strategy.merge.IMergeStrategy)
     */
    @Override
    public int deduplicate(Class<? extends Classification> clazz, IMatchStrategy matchStrategy, IMergeStrategy mergeStrategy) {
        return defaultService.deduplicate(clazz, matchStrategy, mergeStrategy);
    }

    /**
     * @param clazz
     * @param queryString
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param matchMode
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findTitleCache(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, eu.etaxonomy.cdm.persistence.query.MatchMode)
     */
    @Override
    public Pager<Classification> findTitleCache(Class<? extends Classification> clazz, String queryString, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, MatchMode matchMode) {
        return defaultService.findTitleCache(clazz, queryString, pageSize, pageNumber, orderHints, matchMode);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IClassificationService#listChildNodesOfTaxon(java.util.UUID, java.util.UUID, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public List<TaxonNode> listChildNodesOfTaxon(UUID taxonUuid, UUID classificationUuid, Integer pageSize,
            Integer pageIndex, List<String> propertyPaths) {
        return defaultService.listChildNodesOfTaxon(taxonUuid, classificationUuid, pageSize, pageIndex, propertyPaths);
    }

	@Override
	public DeleteResult isDeletable(Classification object,
			DeleteConfiguratorBase config) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Classification findWithoutFlush(UUID uuid) {
		return defaultService.findWithoutFlush(uuid);
	}


	@Override
	public <S extends Classification>  Pager<FindByIdentifierDTO<S>> findByIdentifier(Class<S> clazz, String identifier, DefinedTerm identifierType, MatchMode matchmode, boolean includeEntity, Integer pageSize, Integer pageNumber, List<String> propertyPaths){
		return defaultService.findByIdentifier(clazz, identifier, identifierType, matchmode, includeEntity, pageSize, pageNumber, propertyPaths);
	}

    @Override
    public UpdateResult createHierarchyInClassification(Classification arg1, CreateHierarchyForClassificationConfigurator arg2) {
       return defaultService.createHierarchyInClassification(arg1, arg2);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IService#delete(java.util.UUID)
     */
    @Override
    public DeleteResult delete(UUID arg0) {
        return defaultService.delete(arg0);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IService#delete(java.util.Collection)
     */
    @Override
    public DeleteResult delete(Collection<UUID> arg0) {
        return defaultService.delete(arg0);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IService#merge(java.util.List)
     */
    @Override
    public List<Classification> merge(List<Classification> arg0) {
        return defaultService.merge(arg0);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IService#merge(java.util.List, boolean)
     */
    @Override
    public List<MergeResult<Classification>> merge(List<Classification> arg0, boolean arg1) {
        return defaultService.merge(arg0, arg1);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IService#merge(eu.etaxonomy.cdm.model.common.ICdmBase, boolean)
     */
    @Override
    public MergeResult<Classification> merge(Classification arg0, boolean arg1) {
        return defaultService.merge(arg0, arg1);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IService#loadWithUpdate(java.util.UUID)
     */
    @Override
    public Classification loadWithUpdate(UUID arg0) {
        return defaultService.loadWithUpdate(arg0);
    }


}
