/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.MoveDescriptionToOtherTaxonOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonNodeSelectionDialog;

/**
 * <p>ChangeAcceptedTaxonToSynonymHandler class.</p>
 *
 * @author a.kohlbecker
 * @created Okt. 11, 2013
 * @version 1.0
 *
 */
public class MoveDescriptionToOtherTaxonHandler extends AbstractHandler
		implements IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(MoveDescriptionToOtherTaxonHandler.class);
	private MoveDescriptionToOtherTaxonOperation operation;

	private UUID newAcceptedTaxonNodeUuid;
	private MultiPageTaxonEditor editor;

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	@Override
    public Object execute(ExecutionEvent event) throws ExecutionException {

		ISelection selection = HandlerUtil.getCurrentSelection(event);
		editor = EditorUtil.getActiveMultiPageTaxonEditor();
		if (this.editor.isDirty()){
		    boolean proceed = MessageDialog.openQuestion(null,
	                   "Save changes", "You have made changes that must be saved before this query can be executed. Would you like to proceed?");
	        if (proceed) {
	            editor.doSave(EditorUtil.getMonitor());
	        } else {
	            return null;
	        }

		}


		if(selection instanceof IStructuredSelection){

			List<TaxonDescription> descriptions = new ArrayList<TaxonDescription>();

			IStructuredSelection structuredSelection = (IStructuredSelection) selection;

			for(Object element : structuredSelection.toArray()){
				UUID uuid = null;
				if (element instanceof FeatureNodeContainer){
					uuid = ((FeatureNodeContainer)element).getDescription().getUuid();
				}	else if (element instanceof DescriptionBase){
					uuid = ((DescriptionBase)element).getUuid();
				}

				if (uuid != null){
					descriptions.add((TaxonDescription) CdmStore.getService(IDescriptionService.class).load(uuid, null));
				}
			}
			if(descriptions.size() == 0){
				return null;
			}

			// Choose the target taxon
			List<UUID> excludeTaxa = new ArrayList<UUID>();


			excludeTaxa.add(descriptions.get(0).getTaxon().getUuid());

			//get current taxon node
			TaxonNode node = null;
			Classification classification = null;
			MultiPageTaxonEditor taxonEditor = EditorUtil.getActiveMultiPageTaxonEditor();
            if(taxonEditor!=null){
                node = ((TaxonEditorInput) taxonEditor.getEditorInput()).getTaxonNode();
                classification = node.getClassification();
            }
			TaxonNode newAcceptedTaxonNode = TaxonNodeSelectionDialog.select(HandlerUtil.getActiveShell(event),
					editor.getConversationHolder(),
					"Choose the accepted taxon",
					excludeTaxa,
					node,
					classification);

			if (newAcceptedTaxonNode == null) {
				return null;
			}

			newAcceptedTaxonNodeUuid = newAcceptedTaxonNode.getUuid();

			try {
				for(TaxonDescription description : descriptions){
					operation = new MoveDescriptionToOtherTaxonOperation(event.getCommand().getName(),
											editor.getUndoContext(),
											description,
											newAcceptedTaxonNode,
											this,
											editor,
											(ICdmEntitySessionEnabled)editor.getEditorInput());
					AbstractUtility.executeOperation(operation);
				}

			} catch (NotDefinedException e) {
				logger.warn("Command name not set");
			}
		}

		return null;

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	/** {@inheritDoc} */
	@Override
    public boolean postOperation(CdmBase objectAffectedByOperation) {
		Display.getDefault().asyncExec(new Runnable(){

			@Override
            public void run() {
				AbstractUtility.close(editor);

				try {
					MultiPageTaxonEditor possibleOpenEditor = (MultiPageTaxonEditor) EditorUtil.findEditorByTaxonNodeUuid(newAcceptedTaxonNodeUuid);
					if(possibleOpenEditor != null){
						AbstractUtility.close(possibleOpenEditor);
					}
					EditorUtil.openTaxonNode(newAcceptedTaxonNodeUuid);
				} catch (PartInitException e) {
					MessagingUtils.error(this.getClass(), e);
					throw new RuntimeException(e);
				} catch (Exception e) {
					MessagingUtils.warningDialog("Could not create Taxon", this, e.getMessage());
				}
			}

		});


		return true;
	}

	/**
	 * <p>onComplete</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
}
